from pieces_os_client.models.conversation import Conversation
from ..streamed_identifiers import ConversationsSnapshot
from typing import Optional, List
from .basic import Basic
from .message import BasicMessage

class BasicChat(Basic):
    """
    A class to represent a basic chat, initialized with a conversation ID.
    """
    @property
    def conversation(self) -> Conversation:
        conversation = ConversationsSnapshot.identifiers_snapshot.get(self._id)
        if not conversation:
            raise ValueError("Conversation not found")
        return conversation

    @property
    def id(self):
        """
        Gets the ID of the conversation.

        Returns:
            The ID of the conversation.
        """
        return self.conversation.id

    @property
    def name(self) -> str:
        """
        Gets the name of the conversation.

        Returns:
            The name of the conversation, or "New Conversation" if the name is not set.
        """
        return self.conversation.name if self.conversation.name else "New Conversation"

    @name.setter
    def name(self, name):
        """
        Sets the name of the conversation.

        Args:
            name: The new name of the conversation.
        """
        self.conversation.name = name
        self._edit_conversation(self.conversation)

    def messages(self) -> List[BasicMessage]:
        """
        Retrieves the messages in the conversation.

        Returns:
            A list of BasicMessage instances representing the messages in the conversation.
        """
        out = []
        for message_id, index in (self.conversation.messages.indices or {}).items():
            if index == -1:  # Deleted message
                continue
            out.append(BasicMessage(ConversationsSnapshot.pieces_client,message_id))
        return out


    @property
    def annotations(self):
        """
        Gets the annotations of the conversation.

        Returns:
            The annotations of the conversation, or None if not available.
        """
        return getattr(self.conversation.annotations, "iterable", None)

    def delete(self):
        """
        Deletes the conversation.
        """
        ConversationsSnapshot.pieces_client.conversations_api.conversations_delete_specific_conversation(self.id)

    @staticmethod
    def _edit_conversation(conversation):
        """
        Edits the conversation.

        Args:
            conversation: The conversation to edit.
        """
        ConversationsSnapshot.pieces_client.conversation_api.conversation_update(False, conversation)




# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ModelFoundationEnum(str, Enum):
    """
    This is used to describe the foundational models used within POS.  Note: LATEST models could be used as the latests of these and then could use a system to either(check a file) or       we could add and endpoint to click out to the cloud to get the latest models and update these locally so that our users       can always have the latest without having to update the application.(not used for now)
    """

    """
    allowed enum values
    """
    GPT_3_DOT_5 = 'GPT_3.5'
    GPT_4 = 'GPT_4'
    T5 = 'T5'
    LLAMA_2_7_B = 'LLAMA_2_7B'
    LLAMA_2_13_B = 'LLAMA_2_13B'
    LLAMA_2_70_B = 'LLAMA_2_70B'
    LLAMA_3_2_B = 'LLAMA_3_2B'
    LLAMA_3_13_B = 'LLAMA_3_13B'
    LLAMA_3_70_B = 'LLAMA_3_70B'
    CODE_LLAMA_2_7_B = 'CODE_LLAMA_2_7B'
    CODE_LLAMA_2_13_B = 'CODE_LLAMA_2_13B'
    CODE_LLAMA_2_70_B = 'CODE_LLAMA_2_70B'
    BARD = 'BARD'
    ALPACA_7_B = 'ALPACA_7B'
    ALPACA_13_B = 'ALPACA_13B'
    ALPACA_33_B = 'ALPACA_33B'
    ALPACA_65_B = 'ALPACA_65B'
    VICUNA_7_B = 'VICUNA_7B'
    VICUNA_13_B = 'VICUNA_13B'
    VICUNA_33_B = 'VICUNA_33B'
    VICUNA_65_B = 'VICUNA_65B'
    GUANACO_7_B = 'GUANACO_7B'
    GUANACO_13_B = 'GUANACO_13B'
    GUANACO_33_B = 'GUANACO_33B'
    GUANACO_65_B = 'GUANACO_65B'
    OPENLLAMA_7_B = 'OPENLLAMA_7B'
    OPENLLAMA_13_B = 'OPENLLAMA_13B'
    GORILLA_7_B = 'GORILLA_7B'
    GORILLA_13_B = 'GORILLA_13B'
    GORILLA_33_B = 'GORILLA_33B'
    GORILLA_65_B = 'GORILLA_65B'
    WIZARDLM_7_B = 'WIZARDLM_7B'
    WIZARDLM_13_B = 'WIZARDLM_13B'
    WIZARDLM_30_B = 'WIZARDLM_30B'
    YULANCHAT_13_B = 'YULANCHAT_13B'
    YULANCHAT_65_B = 'YULANCHAT_65B'
    REDPAJAMA_3_B = 'REDPAJAMA_3B'
    REDPAJAMA_7_B = 'REDPAJAMA_7B'
    REDPAJAMA_13_B = 'REDPAJAMA_13B'
    REDPAJAMA_33_B = 'REDPAJAMA_33B'
    REDPAJAMA_65_B = 'REDPAJAMA_65B'
    DOLLY_70_M = 'DOLLY_70M'
    DOLLY_160_M = 'DOLLY_160M'
    DOLLY_410_M = 'DOLLY_410M'
    DOLLY_1_B = 'DOLLY_1B'
    DOLLY_1_DOT_4_B = 'DOLLY_1.4B'
    DOLLY_2_DOT_8_B = 'DOLLY_2.8B'
    DOLLY_6_DOT_9_B = 'DOLLY_6.9B'
    DOLLY_12_B = 'DOLLY_12B'
    PYTHIA_70_M = 'PYTHIA_70M'
    PYTHIA_160_M = 'PYTHIA_160M'
    PYTHIA_410_M = 'PYTHIA_410M'
    PYTHIA_1_B = 'PYTHIA_1B'
    PYTHIA_1_DOT_4_B = 'PYTHIA_1.4B'
    PYTHIA_2_DOT_8_B = 'PYTHIA_2.8B'
    PYTHIA_6_DOT_9_B = 'PYTHIA_6.9B'
    PYTHIA_12_B = 'PYTHIA_12B'
    MOSS_7_B = 'MOSS_7B'
    MOSS_13_B = 'MOSS_13B'
    RMKV_LM_100_M = 'RMKV_LM_100M'
    RMKV_LM_400_M = 'RMKV_LM_400M'
    RMKV_LM_1_DOT_5_B = 'RMKV_LM_1.5B'
    RMKV_LM_3_B = 'RMKV_LM_3B'
    RMKV_LM_7_B = 'RMKV_LM_7B'
    RMKV_LM_14_B = 'RMKV_LM_14B'
    STARCODER_15_DOT_5_B = 'STARCODER_15.5B'
    WIZARDCODER_15_B = 'WIZARDCODER_15B'
    SANTACODER_1_DOT_1_B = 'SANTACODER_1.1B'
    TEXT_BISON = 'TEXT_BISON'
    TEXTEMBEDDING_GECKO = 'TEXTEMBEDDING_GECKO'
    CHAT_BISON = 'CHAT_BISON'
    CODE_BISON = 'CODE_BISON'
    CODECHAT_BISON = 'CODECHAT_BISON'
    CODE_GECKO = 'CODE_GECKO'
    DISTILROBERTA = 'DISTILROBERTA'
    MISTRAL_7_B = 'MISTRAL_7B'
    GEMINI = 'GEMINI'
    GEMINI_1_DOT_5 = 'GEMINI_1.5'
    FAST_TEXT = 'FAST_TEXT'
    UNIXCODER = 'UNIXCODER'
    PHI_1_DOT_5 = 'PHI_1.5'
    PHI_2 = 'PHI_2'
    MIXTRAL_8_X_7_B = 'MIXTRAL_8_x_7B'
    MIXTRAL_8_X_22_B = 'MIXTRAL_8_x_22B'
    GEMMA_7_B = 'GEMMA_7B'
    GEMMA_2_B = 'GEMMA_2B'
    CODE_GEMMA_7_B = 'CODE_GEMMA_7B'
    CODE_GEMMA_2_B = 'CODE_GEMMA_2B'
    RECURRENT_GEMMA_2_B = 'RECURRENT_GEMMA_2B'
    GROK_1 = 'GROK_1'
    GROK_1_DOT_5 = 'GROK_1.5'
    CLAUDE_2 = 'CLAUDE_2'
    CLAUDE_3 = 'CLAUDE_3'
    STARLING_7_B = 'STARLING_7B'
    DBRX = 'DBRX'
    COMMAND_R_PLUS = 'COMMAND_R+'
    GEMMA_1_DOT_1_2_B = 'GEMMA_1.1_2B'
    GEMMA_1_DOT_1_7_B = 'GEMMA_1.1_7B'
    GEMMA_2_9_B = 'GEMMA_2_9B'
    PHI_3_MINI = 'PHI_3_MINI'
    PHI_3_SMALL = 'PHI_3_SMALL'
    GRANITE_3_B = 'GRANITE_3B'
    GRANITE_8_B = 'GRANITE_8B'
    LLAMA_3_8_B = 'LLAMA_3_8B'
    CLAUDE_3_DOT_5 = 'CLAUDE_3.5'
    CLAUDE_LATEST = 'CLAUDE_LATEST'
    GRANITE_LATEST = 'GRANITE_LATEST'
    LLAMA_LATEST = 'LLAMA_LATEST'
    PHI_LATEST = 'PHI_LATEST'
    GEMMA_LATEST = 'GEMMA_LATEST'
    GEMINI_LATEST = 'GEMINI_LATEST'
    GPT_LATEST = 'GPT_LATEST'
    AZURE_LATEST = 'AZURE_LATEST'
    AZURE_FAST = 'AZURE_FAST'
    AZURE_BEST = 'AZURE_BEST'
    AZURE_DEFAULT = 'AZURE_DEFAULT'
    AZURE_CUSTOM = 'AZURE_CUSTOM'
    PERPLEXITY_LATEST = 'PERPLEXITY_LATEST'
    PERPLEXITY_FAST = 'PERPLEXITY_FAST'
    PERPLEXITY_BEST = 'PERPLEXITY_BEST'
    PERPLEXITY_DEFAULT = 'PERPLEXITY_DEFAULT'
    PERPLEXITY_CUSTOM = 'PERPLEXITY_CUSTOM'

    @classmethod
    def from_json(cls, json_str: str) -> ModelFoundationEnum:
        """Create an instance of ModelFoundationEnum from a JSON string"""
        return ModelFoundationEnum(json.loads(json_str))





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class LanguageServerProtocolSeverityEnum(str, Enum):
    """
    modeled of of https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#diagnosticSeverity
    """

    """
    allowed enum values
    """
    ERROR = 'ERROR'
    WARNING = 'WARNING'
    INFORMATION = 'INFORMATION'
    HINT = 'HINT'

    @classmethod
    def from_json(cls, json_str: str) -> LanguageServerProtocolSeverityEnum:
        """Create an instance of LanguageServerProtocolSeverityEnum from a JSON string"""
        return LanguageServerProtocolSeverityEnum(json.loads(json_str))





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from pieces_os_client.models.access_enum import AccessEnum
from pieces_os_client.models.asset import Asset
from pieces_os_client.models.assets import Assets
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.seeded_user import SeededUser

class SeededShare(BaseModel):
    """
     required to pass in an asset or assets.  # noqa: E501
    """
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    asset: Optional[Asset] = None
    users: Optional[conlist(SeededUser)] = Field(default=None, description="if private please specificy some users you want to share this with.")
    access: AccessEnum = Field(...)
    assets: Optional[Assets] = None
    name: Optional[StrictStr] = Field(default=None, description="optional name, if it is available. and must be unique.")
    __properties = ["schema", "asset", "users", "access", "assets", "name"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SeededShare:
        """Create an instance of SeededShare from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of asset
        if self.asset:
            _dict['asset'] = self.asset.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in users (list)
        _items = []
        if self.users:
            for _item in self.users:
                if _item:
                    _items.append(_item.to_dict())
            _dict['users'] = _items
        # override the default output from pydantic by calling `to_dict()` of assets
        if self.assets:
            _dict['assets'] = self.assets.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SeededShare:
        """Create an instance of SeededShare from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SeededShare.parse_obj(obj)

        _obj = SeededShare.parse_obj({
            "var_schema": EmbeddedModelSchema.from_dict(obj.get("schema")) if obj.get("schema") is not None else None,
            "asset": Asset.from_dict(obj.get("asset")) if obj.get("asset") is not None else None,
            "users": [SeededUser.from_dict(_item) for _item in obj.get("users")] if obj.get("users") is not None else None,
            "access": obj.get("access"),
            "assets": Assets.from_dict(obj.get("assets")) if obj.get("assets") is not None else None,
            "name": obj.get("name")
        })
        return _obj





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from pieces_os_client.models.application import Application
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.referenced_workstream_summary import ReferencedWorkstreamSummary
from pieces_os_client.models.score import Score
from pieces_os_client.models.workstream_event_context import WorkstreamEventContext
from pieces_os_client.models.workstream_event_trigger import WorkstreamEventTrigger

class SeededWorkstreamEvent(BaseModel):
    """
    This is a precreated version of a WorkstreamEvent event, this will be used ingested into PiecesOS and PiecesOS will do all the magic to transform this into relevant data show in the workstream feed.  # noqa: E501
    """
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    score: Optional[Score] = None
    application: Application = Field(...)
    trigger: WorkstreamEventTrigger = Field(...)
    context: Optional[WorkstreamEventContext] = None
    summary: Optional[ReferencedWorkstreamSummary] = None
    internal_identifier: Optional[StrictStr] = Field(default=None, description="This is used to override the event identifier, if this was an event that was originally in the internal events collection.")
    __properties = ["schema", "score", "application", "trigger", "context", "summary", "internal_identifier"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SeededWorkstreamEvent:
        """Create an instance of SeededWorkstreamEvent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of score
        if self.score:
            _dict['score'] = self.score.to_dict()
        # override the default output from pydantic by calling `to_dict()` of application
        if self.application:
            _dict['application'] = self.application.to_dict()
        # override the default output from pydantic by calling `to_dict()` of trigger
        if self.trigger:
            _dict['trigger'] = self.trigger.to_dict()
        # override the default output from pydantic by calling `to_dict()` of context
        if self.context:
            _dict['context'] = self.context.to_dict()
        # override the default output from pydantic by calling `to_dict()` of summary
        if self.summary:
            _dict['summary'] = self.summary.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SeededWorkstreamEvent:
        """Create an instance of SeededWorkstreamEvent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SeededWorkstreamEvent.parse_obj(obj)

        _obj = SeededWorkstreamEvent.parse_obj({
            "var_schema": EmbeddedModelSchema.from_dict(obj.get("schema")) if obj.get("schema") is not None else None,
            "score": Score.from_dict(obj.get("score")) if obj.get("score") is not None else None,
            "application": Application.from_dict(obj.get("application")) if obj.get("application") is not None else None,
            "trigger": WorkstreamEventTrigger.from_dict(obj.get("trigger")) if obj.get("trigger") is not None else None,
            "context": WorkstreamEventContext.from_dict(obj.get("context")) if obj.get("context") is not None else None,
            "summary": ReferencedWorkstreamSummary.from_dict(obj.get("summary")) if obj.get("summary") is not None else None,
            "internal_identifier": obj.get("internal_identifier")
        })
        return _obj





from pieces_os_client import StreamedIdentifiers, Conversation
from .base_websocket import BaseWebsocket
from ..streamed_identifiers import ConversationsSnapshot
from websocket import WebSocketApp
from typing import Optional, Callable,TYPE_CHECKING

if TYPE_CHECKING:
	from ..client import PiecesClient

class ConversationWS(BaseWebsocket):
	def __init__(self, pieces_client: "PiecesClient", 
				 on_conversation_update: Optional[Callable[[Conversation], None]] = None,
				 on_conversation_remove: Optional[Callable[[Conversation], None]] = None,
				 on_open_callback: Optional[Callable[[WebSocketApp], None]] = None, 
				 on_error: Optional[Callable[[WebSocketApp, Exception], None]] = None, 
				 on_close: Optional[Callable[[WebSocketApp], None]] = None):
		"""
		Initialize the ConversationWS class.

		:param pieces_client: An instance of PiecesClient.
		:param on_conversation_update: Optional callback for when a conversation is updated.
		:param on_conversation_remove: Optional callback for when a conversation is removed.
		:param on_open_callback: Optional callback for when the WebSocket connection is opened.
		:param on_error: Optional callback for when an error occurs.
		:param on_close: Optional callback for when the WebSocket connection is closed.
		"""
		# Set the pieces_client for ConversationsSnapshot
		ConversationsSnapshot.pieces_client = pieces_client
		
		# Set the update and remove callbacks, defaulting to no-op lambdas if not provided
		if on_conversation_update:
			ConversationsSnapshot.on_update_list.append(on_conversation_update)
		if on_conversation_remove:
			ConversationsSnapshot.on_remove_list.append(on_conversation_remove)
		
		super().__init__(pieces_client, ConversationsSnapshot.streamed_identifiers_callback, on_open_callback, on_error, on_close)
		ConversationsSnapshot._initialized = self._initialized
		# Initialize the base WebSocket with the provided callbacks
	
	@property
	def url(self):
		"""
		Property to get the WebSocket URL for conversations.

		:return: The WebSocket URL for conversations.
		"""
		return self.pieces_client.CONVERSATION_WS_URL

	def on_message(self, ws, message):
		"""
		Handle incoming WebSocket messages.

		:param ws: The WebSocketApp instance.
		:param message: The incoming message as a JSON string.
		"""
		# Parse the incoming message and pass it to the callback
		self.on_message_callback(StreamedIdentifiers.from_json(message))

	@property
	def _is_initialized_on_open(self):
		return False


# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ClassificationGenericEnum(str, Enum):
    """
    EXTERNAL_RESOURCE_REFERENCE can be thought of as anything such as URL, URI, UNIX PATH, etc
    """

    """
    allowed enum values
    """
    CODE = 'CODE'
    TEXT = 'TEXT'
    IMAGE = 'IMAGE'
    VIDEO = 'VIDEO'
    EXTERNAL_RESOURCE_REFERENCE = 'EXTERNAL_RESOURCE_REFERENCE'
    UNKNOWN = 'UNKNOWN'
    FILE = 'FILE'

    @classmethod
    def from_json(cls, json_str: str) -> ClassificationGenericEnum:
        """Create an instance of ClassificationGenericEnum from a JSON string"""
        return ClassificationGenericEnum(json.loads(json_str))





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.mechanism_enum import MechanismEnum
from pieces_os_client.models.sensitive_category_enum import SensitiveCategoryEnum
from pieces_os_client.models.sensitive_metadata import SensitiveMetadata
from pieces_os_client.models.sensitive_severity_enum import SensitiveSeverityEnum

class SeededSensitive(BaseModel):
    """
    This is the seededSensitive, this does not have an id yet as we will add it on the server side.  can optionally pass in our mechanism here, as the default will be manual unless specified.  TODO consider updating these asset,format to referenced Models  # noqa: E501
    """
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    asset: StrictStr = Field(...)
    text: StrictStr = Field(default=..., description="this is the string representative of the sensative piece of data.")
    mechanism: Optional[MechanismEnum] = None
    category: SensitiveCategoryEnum = Field(...)
    severity: SensitiveSeverityEnum = Field(...)
    name: StrictStr = Field(...)
    description: StrictStr = Field(...)
    metadata: Optional[SensitiveMetadata] = None
    __properties = ["schema", "asset", "text", "mechanism", "category", "severity", "name", "description", "metadata"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SeededSensitive:
        """Create an instance of SeededSensitive from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SeededSensitive:
        """Create an instance of SeededSensitive from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SeededSensitive.parse_obj(obj)

        _obj = SeededSensitive.parse_obj({
            "var_schema": EmbeddedModelSchema.from_dict(obj.get("schema")) if obj.get("schema") is not None else None,
            "asset": obj.get("asset"),
            "text": obj.get("text"),
            "mechanism": obj.get("mechanism"),
            "category": obj.get("category"),
            "severity": obj.get("severity"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "metadata": SensitiveMetadata.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool
from pieces_os_client.models.anonymous_temporal_range import AnonymousTemporalRange
from pieces_os_client.models.classification import Classification
from pieces_os_client.models.document_contributors import DocumentContributors
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.ide_selections import IDESelections
from pieces_os_client.models.language_server_protocol import LanguageServerProtocol
from pieces_os_client.models.transferable_string import TransferableString

class IDETab(BaseModel):
    """
    This is a representation of an IDE Tab  value: is the value of the entire file(that being said we do not recomment passing this over as we can read this file on PieceOS side of things) classification: this is the classifcation of this file  selection: this is a represention of a copy/paste/selection of a bit of code  anchor: this is the file path  range: this is the duration that this user has been on this Tab  current: is a boolean that will let us know if this is the current active tab  contributors: is the people that are extracted via git  lsp: this is the languageserverprotocol this is used for may things such as error,stackstrces, mainly information extracted from the lang server  # noqa: E501
    """
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    value: Optional[TransferableString] = None
    classification: Optional[Classification] = None
    selections: Optional[IDESelections] = None
    anchor: SeededAnchor = Field(...)
    range: Optional[AnonymousTemporalRange] = None
    current: Optional[StrictBool] = None
    contributors: Optional[DocumentContributors] = None
    lsp: Optional[LanguageServerProtocol] = None
    __properties = ["schema", "value", "classification", "selections", "anchor", "range", "current", "contributors", "lsp"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IDETab:
        """Create an instance of IDETab from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of value
        if self.value:
            _dict['value'] = self.value.to_dict()
        # override the default output from pydantic by calling `to_dict()` of classification
        if self.classification:
            _dict['classification'] = self.classification.to_dict()
        # override the default output from pydantic by calling `to_dict()` of selections
        if self.selections:
            _dict['selections'] = self.selections.to_dict()
        # override the default output from pydantic by calling `to_dict()` of anchor
        if self.anchor:
            _dict['anchor'] = self.anchor.to_dict()
        # override the default output from pydantic by calling `to_dict()` of range
        if self.range:
            _dict['range'] = self.range.to_dict()
        # override the default output from pydantic by calling `to_dict()` of contributors
        if self.contributors:
            _dict['contributors'] = self.contributors.to_dict()
        # override the default output from pydantic by calling `to_dict()` of lsp
        if self.lsp:
            _dict['lsp'] = self.lsp.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> IDETab:
        """Create an instance of IDETab from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return IDETab.parse_obj(obj)

        _obj = IDETab.parse_obj({
            "var_schema": EmbeddedModelSchema.from_dict(obj.get("schema")) if obj.get("schema") is not None else None,
            "value": TransferableString.from_dict(obj.get("value")) if obj.get("value") is not None else None,
            "classification": Classification.from_dict(obj.get("classification")) if obj.get("classification") is not None else None,
            "selections": IDESelections.from_dict(obj.get("selections")) if obj.get("selections") is not None else None,
            "anchor": SeededAnchor.from_dict(obj.get("anchor")) if obj.get("anchor") is not None else None,
            "range": AnonymousTemporalRange.from_dict(obj.get("range")) if obj.get("range") is not None else None,
            "current": obj.get("current"),
            "contributors": DocumentContributors.from_dict(obj.get("contributors")) if obj.get("contributors") is not None else None,
            "lsp": LanguageServerProtocol.from_dict(obj.get("lsp")) if obj.get("lsp") is not None else None
        })
        return _obj

from pieces_os_client.models.seeded_anchor import SeededAnchor
IDETab.update_forward_refs()




# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class MechanismEnum(str, Enum):
    """
    This will let us know how an asset was uploaded.  MANUAL: this is an event such as a user driven send to pieces, or a paste to pieces.  RECOMMENDED: This is an event that was initiall recommended via the suggestion flow, and a user decided totake the recommendation.  AUTOMATIC: This is not user driven at all and the asset was automatically uploaded via the suggestion flow. or other methods.  INTERNAL: This is an undelete able enum used for internal use only.
    """

    """
    allowed enum values
    """
    MANUAL = 'MANUAL'
    RECOMMENDED = 'RECOMMENDED'
    AUTOMATIC = 'AUTOMATIC'
    INTERNAL = 'INTERNAL'
    DELETED = 'DELETED'

    @classmethod
    def from_json(cls, json_str: str) -> MechanismEnum:
        """Create an instance of MechanismEnum from a JSON string"""
        return MechanismEnum(json.loads(json_str))





# coding: utf-8

# flake8: noqa
"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from pieces_os_client.models.access_enum import AccessEnum
from pieces_os_client.models.accessor import Accessor
from pieces_os_client.models.accessors import Accessors
from pieces_os_client.models.active_os_server_applet import ActiveOSServerApplet
from pieces_os_client.models.activities import Activities
from pieces_os_client.models.activity import Activity
from pieces_os_client.models.aesthetics import Aesthetics
from pieces_os_client.models.allocation_cloud import AllocationCloud
from pieces_os_client.models.allocation_cloud_status import AllocationCloudStatus
from pieces_os_client.models.allocation_cloud_url import AllocationCloudUrl
from pieces_os_client.models.allocation_cloud_urls import AllocationCloudUrls
from pieces_os_client.models.allocation_status_enum import AllocationStatusEnum
from pieces_os_client.models.allocations import Allocations
from pieces_os_client.models.analyses import Analyses
from pieces_os_client.models.analysis import Analysis
from pieces_os_client.models.analytics_tracked_adoption_event_identifier_description_pairs import AnalyticsTrackedAdoptionEventIdentifierDescriptionPairs
from pieces_os_client.models.anchor import Anchor
from pieces_os_client.models.anchor_point import AnchorPoint
from pieces_os_client.models.anchor_points import AnchorPoints
from pieces_os_client.models.anchor_type_enum import AnchorTypeEnum
from pieces_os_client.models.anchors import Anchors
from pieces_os_client.models.annotation import Annotation
from pieces_os_client.models.annotation_type_enum import AnnotationTypeEnum
from pieces_os_client.models.annotations import Annotations
from pieces_os_client.models.anonymous_temporal_range import AnonymousTemporalRange
from pieces_os_client.models.application import Application
from pieces_os_client.models.application_name_enum import ApplicationNameEnum
from pieces_os_client.models.applications import Applications
from pieces_os_client.models.asset import Asset
from pieces_os_client.models.asset_filter import AssetFilter
from pieces_os_client.models.asset_filter_phrase import AssetFilterPhrase
from pieces_os_client.models.asset_filter_phrase_options import AssetFilterPhraseOptions
from pieces_os_client.models.asset_filter_timestamp import AssetFilterTimestamp
from pieces_os_client.models.asset_filters import AssetFilters
from pieces_os_client.models.asset_reclassification import AssetReclassification
from pieces_os_client.models.asset_search_space import AssetSearchSpace
from pieces_os_client.models.assets import Assets
from pieces_os_client.models.assets_search_with_filters_input import AssetsSearchWithFiltersInput
from pieces_os_client.models.assets_search_with_filters_output import AssetsSearchWithFiltersOutput
from pieces_os_client.models.auth0 import Auth0
from pieces_os_client.models.auth0_identity import Auth0Identity
from pieces_os_client.models.auth0_open_ai_user_metadata import Auth0OpenAIUserMetadata
from pieces_os_client.models.auth0_redirects import Auth0Redirects
from pieces_os_client.models.auth0_user import Auth0User
from pieces_os_client.models.auth0_user_allocation_metadata import Auth0UserAllocationMetadata
from pieces_os_client.models.auth0_user_beta_status import Auth0UserBetaStatus
from pieces_os_client.models.auth0_user_metadata import Auth0UserMetadata
from pieces_os_client.models.available_formats import AvailableFormats
from pieces_os_client.models.backup import Backup
from pieces_os_client.models.backup_status import BackupStatus
from pieces_os_client.models.backup_status_enum import BackupStatusEnum
from pieces_os_client.models.backup_streamed_progress import BackupStreamedProgress
from pieces_os_client.models.backups import Backups
from pieces_os_client.models.backups_streamed_progress import BackupsStreamedProgress
from pieces_os_client.models.browser_selection import BrowserSelection
from pieces_os_client.models.browser_tab import BrowserTab
from pieces_os_client.models.browser_tab_value import BrowserTabValue
from pieces_os_client.models.browser_tab_values import BrowserTabValues
from pieces_os_client.models.browser_tabs import BrowserTabs
from pieces_os_client.models.byte_descriptor import ByteDescriptor
from pieces_os_client.models.capabilities_enum import CapabilitiesEnum
from pieces_os_client.models.challenged_pkce import ChallengedPKCE
from pieces_os_client.models.classification import Classification
from pieces_os_client.models.classification_generic_enum import ClassificationGenericEnum
from pieces_os_client.models.classification_rendering_enum import ClassificationRenderingEnum
from pieces_os_client.models.classification_specific_enum import ClassificationSpecificEnum
from pieces_os_client.models.classifications import Classifications
from pieces_os_client.models.code_analyses import CodeAnalyses
from pieces_os_client.models.code_analysis import CodeAnalysis
from pieces_os_client.models.context import Context
from pieces_os_client.models.conversation import Conversation
from pieces_os_client.models.conversation_grounding import ConversationGrounding
from pieces_os_client.models.conversation_message import ConversationMessage
from pieces_os_client.models.conversation_message_sentiment_enum import ConversationMessageSentimentEnum
from pieces_os_client.models.conversation_messages import ConversationMessages
from pieces_os_client.models.conversation_summarize_input import ConversationSummarizeInput
from pieces_os_client.models.conversation_summarize_output import ConversationSummarizeOutput
from pieces_os_client.models.conversation_type_enum import ConversationTypeEnum
from pieces_os_client.models.conversations import Conversations
from pieces_os_client.models.conversations_create_from_asset_output import ConversationsCreateFromAssetOutput
from pieces_os_client.models.created_external_provider_api_key import CreatedExternalProviderApiKey
from pieces_os_client.models.deleted_external_provider_api_key import DeletedExternalProviderApiKey
from pieces_os_client.models.detected_external_application import DetectedExternalApplication
from pieces_os_client.models.detected_external_applications import DetectedExternalApplications
from pieces_os_client.models.discovered_asset import DiscoveredAsset
from pieces_os_client.models.discovered_assets import DiscoveredAssets
from pieces_os_client.models.discovered_html_webpage import DiscoveredHtmlWebpage
from pieces_os_client.models.discovered_html_webpages import DiscoveredHtmlWebpages
from pieces_os_client.models.discovered_related_tag import DiscoveredRelatedTag
from pieces_os_client.models.discovered_related_tags import DiscoveredRelatedTags
from pieces_os_client.models.discovered_sensitive import DiscoveredSensitive
from pieces_os_client.models.discovered_sensitives import DiscoveredSensitives
from pieces_os_client.models.distribution import Distribution
from pieces_os_client.models.distributions import Distributions
from pieces_os_client.models.document_contributor import DocumentContributor
from pieces_os_client.models.document_contributors import DocumentContributors
from pieces_os_client.models.edges import Edges
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.embedded_model_schema_semantic_version_enum import EmbeddedModelSchemaSemanticVersionEnum
from pieces_os_client.models.embedding import Embedding
from pieces_os_client.models.embeddings import Embeddings
from pieces_os_client.models.embeddings_search_options import EmbeddingsSearchOptions
from pieces_os_client.models.embeddings_search_options_embedding_type_enum import EmbeddingsSearchOptionsEmbeddingTypeEnum
from pieces_os_client.models.existent_metadata import ExistentMetadata
from pieces_os_client.models.existing_metadata import ExistingMetadata
from pieces_os_client.models.exported_asset import ExportedAsset
from pieces_os_client.models.exported_database import ExportedDatabase
from pieces_os_client.models.exported_database_format import ExportedDatabaseFormat
from pieces_os_client.models.exported_database_formats import ExportedDatabaseFormats
from pieces_os_client.models.external_ml_provider_enum import ExternalMLProviderEnum
from pieces_os_client.models.external_provider import ExternalProvider
from pieces_os_client.models.external_provider_profile_data import ExternalProviderProfileData
from pieces_os_client.models.external_provider_type_enum import ExternalProviderTypeEnum
from pieces_os_client.models.external_providers import ExternalProviders
from pieces_os_client.models.externally_sourced_enum import ExternallySourcedEnum
from pieces_os_client.models.file_format import FileFormat
from pieces_os_client.models.file_metadata import FileMetadata
from pieces_os_client.models.file_picker_input import FilePickerInput
from pieces_os_client.models.filter_operation_type_enum import FilterOperationTypeEnum
from pieces_os_client.models.flattened_activities import FlattenedActivities
from pieces_os_client.models.flattened_activity import FlattenedActivity
from pieces_os_client.models.flattened_analysis import FlattenedAnalysis
from pieces_os_client.models.flattened_anchor import FlattenedAnchor
from pieces_os_client.models.flattened_anchor_point import FlattenedAnchorPoint
from pieces_os_client.models.flattened_anchor_points import FlattenedAnchorPoints
from pieces_os_client.models.flattened_anchors import FlattenedAnchors
from pieces_os_client.models.flattened_annotation import FlattenedAnnotation
from pieces_os_client.models.flattened_annotations import FlattenedAnnotations
from pieces_os_client.models.flattened_application import FlattenedApplication
from pieces_os_client.models.flattened_asset import FlattenedAsset
from pieces_os_client.models.flattened_assets import FlattenedAssets
from pieces_os_client.models.flattened_conversation import FlattenedConversation
from pieces_os_client.models.flattened_conversation_message import FlattenedConversationMessage
from pieces_os_client.models.flattened_conversation_messages import FlattenedConversationMessages
from pieces_os_client.models.flattened_conversations import FlattenedConversations
from pieces_os_client.models.flattened_distribution import FlattenedDistribution
from pieces_os_client.models.flattened_distributions import FlattenedDistributions
from pieces_os_client.models.flattened_format import FlattenedFormat
from pieces_os_client.models.flattened_formats import FlattenedFormats
from pieces_os_client.models.flattened_hint import FlattenedHint
from pieces_os_client.models.flattened_hints import FlattenedHints
from pieces_os_client.models.flattened_image_analysis import FlattenedImageAnalysis
from pieces_os_client.models.flattened_ocr_analysis import FlattenedOCRAnalysis
from pieces_os_client.models.flattened_person import FlattenedPerson
from pieces_os_client.models.flattened_persons import FlattenedPersons
from pieces_os_client.models.flattened_preview import FlattenedPreview
from pieces_os_client.models.flattened_range import FlattenedRange
from pieces_os_client.models.flattened_ranges import FlattenedRanges
from pieces_os_client.models.flattened_sensitive import FlattenedSensitive
from pieces_os_client.models.flattened_sensitives import FlattenedSensitives
from pieces_os_client.models.flattened_share import FlattenedShare
from pieces_os_client.models.flattened_shares import FlattenedShares
from pieces_os_client.models.flattened_tag import FlattenedTag
from pieces_os_client.models.flattened_tags import FlattenedTags
from pieces_os_client.models.flattened_user_profile import FlattenedUserProfile
from pieces_os_client.models.flattened_website import FlattenedWebsite
from pieces_os_client.models.flattened_websites import FlattenedWebsites
from pieces_os_client.models.flattened_workstream_event import FlattenedWorkstreamEvent
from pieces_os_client.models.flattened_workstream_events import FlattenedWorkstreamEvents
from pieces_os_client.models.flattened_workstream_pattern_engine_vision_event import FlattenedWorkstreamPatternEngineVisionEvent
from pieces_os_client.models.flattened_workstream_pattern_engine_vision_events import FlattenedWorkstreamPatternEngineVisionEvents
from pieces_os_client.models.flattened_workstream_summaries import FlattenedWorkstreamSummaries
from pieces_os_client.models.flattened_workstream_summary import FlattenedWorkstreamSummary
from pieces_os_client.models.font import Font
from pieces_os_client.models.format import Format
from pieces_os_client.models.format_metric import FormatMetric
from pieces_os_client.models.format_reclassification import FormatReclassification
from pieces_os_client.models.formats import Formats
from pieces_os_client.models.formats_metrics import FormatsMetrics
from pieces_os_client.models.fragment_format import FragmentFormat
from pieces_os_client.models.fragment_metadata import FragmentMetadata
from pieces_os_client.models.full_text_search_options import FullTextSearchOptions
from pieces_os_client.models.git_hub_distribution import GitHubDistribution
from pieces_os_client.models.git_hub_gist_distribution import GitHubGistDistribution
from pieces_os_client.models.graphical_image_descriptive_statistics import GraphicalImageDescriptiveStatistics
from pieces_os_client.models.graphical_image_processing import GraphicalImageProcessing
from pieces_os_client.models.graphical_image_statistics import GraphicalImageStatistics
from pieces_os_client.models.graphical_machine_learning_processing_event import GraphicalMachineLearningProcessingEvent
from pieces_os_client.models.graphical_ocr_descriptive_statistics import GraphicalOCRDescriptiveStatistics
from pieces_os_client.models.graphical_ocr_descriptive_statistics_confidence import GraphicalOCRDescriptiveStatisticsConfidence
from pieces_os_client.models.graphical_ocr_processing import GraphicalOCRProcessing
from pieces_os_client.models.graphical_ocr_statistics import GraphicalOCRStatistics
from pieces_os_client.models.graphical_svg_statistics import GraphicalSVGStatistics
from pieces_os_client.models.grouped_timestamp import GroupedTimestamp
from pieces_os_client.models.health import Health
from pieces_os_client.models.hint import Hint
from pieces_os_client.models.hint_type_enum import HintTypeEnum
from pieces_os_client.models.hints import Hints
from pieces_os_client.models.ide_selection import IDESelection
from pieces_os_client.models.ide_selections import IDESelections
from pieces_os_client.models.ide_tab import IDETab
from pieces_os_client.models.ide_tabs import IDETabs
from pieces_os_client.models.image_analyses import ImageAnalyses
from pieces_os_client.models.image_analysis import ImageAnalysis
from pieces_os_client.models.inactive_os_server_applet import InactiveOSServerApplet
from pieces_os_client.models.interacted_asset import InteractedAsset
from pieces_os_client.models.interacted_asset_interactions import InteractedAssetInteractions
from pieces_os_client.models.interacted_assets import InteractedAssets
from pieces_os_client.models.language_server_protocol import LanguageServerProtocol
from pieces_os_client.models.language_server_protocol_code import LanguageServerProtocolCode
from pieces_os_client.models.language_server_protocol_code_description import LanguageServerProtocolCodeDescription
from pieces_os_client.models.language_server_protocol_diagnostic import LanguageServerProtocolDiagnostic
from pieces_os_client.models.language_server_protocol_diagnostics import LanguageServerProtocolDiagnostics
from pieces_os_client.models.language_server_protocol_location import LanguageServerProtocolLocation
from pieces_os_client.models.language_server_protocol_location_range import LanguageServerProtocolLocationRange
from pieces_os_client.models.language_server_protocol_location_range_position import LanguageServerProtocolLocationRangePosition
from pieces_os_client.models.language_server_protocol_severity_enum import LanguageServerProtocolSeverityEnum
from pieces_os_client.models.linkify import Linkify
from pieces_os_client.models.linkify_multiple import LinkifyMultiple
from pieces_os_client.models.mailgun_distribution import MailgunDistribution
from pieces_os_client.models.mailgun_metadata import MailgunMetadata
from pieces_os_client.models.mechanism_enum import MechanismEnum
from pieces_os_client.models.model import Model
from pieces_os_client.models.model_capabilities import ModelCapabilities
from pieces_os_client.models.model_delete_cache_input import ModelDeleteCacheInput
from pieces_os_client.models.model_delete_cache_output import ModelDeleteCacheOutput
from pieces_os_client.models.model_download_progress import ModelDownloadProgress
from pieces_os_client.models.model_download_progress_status_enum import ModelDownloadProgressStatusEnum
from pieces_os_client.models.model_foundation_enum import ModelFoundationEnum
from pieces_os_client.models.model_max_tokens import ModelMaxTokens
from pieces_os_client.models.model_type_enum import ModelTypeEnum
from pieces_os_client.models.model_usage_enum import ModelUsageEnum
from pieces_os_client.models.models import Models
from pieces_os_client.models.node import Node
from pieces_os_client.models.node_type_enum import NodeTypeEnum
from pieces_os_client.models.notification import Notification
from pieces_os_client.models.o_auth_account import OAuthAccount
from pieces_os_client.models.o_auth_group import OAuthGroup
from pieces_os_client.models.o_auth_token import OAuthToken
from pieces_os_client.models.ocr_analyses import OCRAnalyses
from pieces_os_client.models.ocr_analysis import OCRAnalysis
from pieces_os_client.models.os_applet_enum import OSAppletEnum
from pieces_os_client.models.os_device_cpu_hardware_information import OSDeviceCPUHardwareInformation
from pieces_os_client.models.os_device_dependencies_information import OSDeviceDependenciesInformation
from pieces_os_client.models.os_device_gpu_hardware_capabilities_information import OSDeviceGPUHardwareCapabilitiesInformation
from pieces_os_client.models.os_device_gpu_hardware_information import OSDeviceGPUHardwareInformation
from pieces_os_client.models.os_device_hardware_information import OSDeviceHardwareInformation
from pieces_os_client.models.os_device_information_returnable import OSDeviceInformationReturnable
from pieces_os_client.models.os_file_streaming_read import OSFileStreamingRead
from pieces_os_client.models.os_file_streaming_read_attempt import OSFileStreamingReadAttempt
from pieces_os_client.models.os_file_streaming_read_progress import OSFileStreamingReadProgress
from pieces_os_client.models.os_health import OSHealth
from pieces_os_client.models.os_permissions import OSPermissions
from pieces_os_client.models.os_processing_permissions import OSProcessingPermissions
from pieces_os_client.models.os_server_metadata import OSServerMetadata
from pieces_os_client.models.os_server_settings import OSServerSettings
from pieces_os_client.models.os_server_update_status import OSServerUpdateStatus
from pieces_os_client.models.onboarded_persona_details import OnboardedPersonaDetails
from pieces_os_client.models.open_ai_models_list_input import OpenAIModelsListInput
from pieces_os_client.models.open_ai_models_list_output import OpenAIModelsListOutput
from pieces_os_client.models.ordered_metrics import OrderedMetrics
from pieces_os_client.models.pkce import PKCE
from pieces_os_client.models.person import Person
from pieces_os_client.models.person_access import PersonAccess
from pieces_os_client.models.person_access_scoped_enum import PersonAccessScopedEnum
from pieces_os_client.models.person_basic_type import PersonBasicType
from pieces_os_client.models.person_model import PersonModel
from pieces_os_client.models.person_type import PersonType
from pieces_os_client.models.persons import Persons
from pieces_os_client.models.platform_enum import PlatformEnum
from pieces_os_client.models.precreated_external_provider_api_key import PrecreatedExternalProviderApiKey
from pieces_os_client.models.predeleted_external_provider_api_key import PredeletedExternalProviderApiKey
from pieces_os_client.models.preonboarded_persona_details import PreonboardedPersonaDetails
from pieces_os_client.models.preupdated_external_provider_api_key import PreupdatedExternalProviderApiKey
from pieces_os_client.models.preview import Preview
from pieces_os_client.models.privacy_enum import PrivacyEnum
from pieces_os_client.models.project_module import ProjectModule
from pieces_os_client.models.project_modules import ProjectModules
from pieces_os_client.models.pseudo_assets import PseudoAssets
from pieces_os_client.models.qgpt_agent_related_routes import QGPTAgentRelatedRoutes
from pieces_os_client.models.qgpt_agent_routes import QGPTAgentRoutes
from pieces_os_client.models.qgpt_conversation import QGPTConversation
from pieces_os_client.models.qgpt_conversation_message import QGPTConversationMessage
from pieces_os_client.models.qgpt_conversation_message_role_enum import QGPTConversationMessageRoleEnum
from pieces_os_client.models.qgpt_conversation_pipeline import QGPTConversationPipeline
from pieces_os_client.models.qgpt_conversation_pipeline_for_contextualized_code_dialog import QGPTConversationPipelineForContextualizedCodeDialog
from pieces_os_client.models.qgpt_conversation_pipeline_for_contextualized_code_generation import QGPTConversationPipelineForContextualizedCodeGeneration
from pieces_os_client.models.qgpt_conversation_pipeline_for_contextualized_code_workstream_dialog import QGPTConversationPipelineForContextualizedCodeWorkstreamDialog
from pieces_os_client.models.qgpt_conversation_pipeline_for_generalized_code_dialog import QGPTConversationPipelineForGeneralizedCodeDialog
from pieces_os_client.models.qgpt_hints_input import QGPTHintsInput
from pieces_os_client.models.qgpt_persons_related_input import QGPTPersonsRelatedInput
from pieces_os_client.models.qgpt_persons_related_output import QGPTPersonsRelatedOutput
from pieces_os_client.models.qgpt_prompt_pipeline import QGPTPromptPipeline
from pieces_os_client.models.qgpt_question_answer import QGPTQuestionAnswer
from pieces_os_client.models.qgpt_question_answers import QGPTQuestionAnswers
from pieces_os_client.models.qgpt_question_input import QGPTQuestionInput
from pieces_os_client.models.qgpt_question_output import QGPTQuestionOutput
from pieces_os_client.models.qgpt_relevance_input import QGPTRelevanceInput
from pieces_os_client.models.qgpt_relevance_input_options import QGPTRelevanceInputOptions
from pieces_os_client.models.qgpt_relevance_output import QGPTRelevanceOutput
from pieces_os_client.models.qgpt_reprompt_input import QGPTRepromptInput
from pieces_os_client.models.qgpt_reprompt_output import QGPTRepromptOutput
from pieces_os_client.models.qgpt_stream_enum import QGPTStreamEnum
from pieces_os_client.models.qgpt_stream_input import QGPTStreamInput
from pieces_os_client.models.qgpt_stream_output import QGPTStreamOutput
from pieces_os_client.models.qgpt_streamed_output_extracted_materials import QGPTStreamedOutputExtractedMaterials
from pieces_os_client.models.qgpt_streamed_output_extracted_textual_materials import QGPTStreamedOutputExtractedTextualMaterials
from pieces_os_client.models.qgpt_task_pipeline import QGPTTaskPipeline
from pieces_os_client.models.qgpt_task_pipeline_for_code_commentation import QGPTTaskPipelineForCodeCommentation
from pieces_os_client.models.qgpt_task_pipeline_for_code_completion import QGPTTaskPipelineForCodeCompletion
from pieces_os_client.models.qgpt_task_pipeline_for_code_explanation import QGPTTaskPipelineForCodeExplanation
from pieces_os_client.models.qgpt_task_pipeline_for_code_fix import QGPTTaskPipelineForCodeFix
from pieces_os_client.models.qgpt_task_pipeline_for_code_modification import QGPTTaskPipelineForCodeModification
from pieces_os_client.models.range import Range
from pieces_os_client.models.ranges import Ranges
from pieces_os_client.models.reaction import Reaction
from pieces_os_client.models.recipients import Recipients
from pieces_os_client.models.referenced_activity import ReferencedActivity
from pieces_os_client.models.referenced_anchor import ReferencedAnchor
from pieces_os_client.models.referenced_anchor_point import ReferencedAnchorPoint
from pieces_os_client.models.referenced_annotation import ReferencedAnnotation
from pieces_os_client.models.referenced_application import ReferencedApplication
from pieces_os_client.models.referenced_asset import ReferencedAsset
from pieces_os_client.models.referenced_conversation import ReferencedConversation
from pieces_os_client.models.referenced_conversation_message import ReferencedConversationMessage
from pieces_os_client.models.referenced_distribution import ReferencedDistribution
from pieces_os_client.models.referenced_format import ReferencedFormat
from pieces_os_client.models.referenced_hint import ReferencedHint
from pieces_os_client.models.referenced_model import ReferencedModel
from pieces_os_client.models.referenced_person import ReferencedPerson
from pieces_os_client.models.referenced_range import ReferencedRange
from pieces_os_client.models.referenced_sensitive import ReferencedSensitive
from pieces_os_client.models.referenced_share import ReferencedShare
from pieces_os_client.models.referenced_tag import ReferencedTag
from pieces_os_client.models.referenced_user import ReferencedUser
from pieces_os_client.models.referenced_website import ReferencedWebsite
from pieces_os_client.models.referenced_workstream_event import ReferencedWorkstreamEvent
from pieces_os_client.models.referenced_workstream_pattern_engine_vision_event import ReferencedWorkstreamPatternEngineVisionEvent
from pieces_os_client.models.referenced_workstream_summary import ReferencedWorkstreamSummary
from pieces_os_client.models.relationship import Relationship
from pieces_os_client.models.relationships import Relationships
from pieces_os_client.models.relevant_qgpt_seed import RelevantQGPTSeed
from pieces_os_client.models.relevant_qgpt_seeds import RelevantQGPTSeeds
from pieces_os_client.models.resulted_pkce import ResultedPKCE
from pieces_os_client.models.returned_user_profile import ReturnedUserProfile
from pieces_os_client.models.reuse_reaction import ReuseReaction
from pieces_os_client.models.reuse_suggestion import ReuseSuggestion
from pieces_os_client.models.revoked_pkce import RevokedPKCE
from pieces_os_client.models.role import Role
from pieces_os_client.models.save_suggestion import SaveSuggestion
from pieces_os_client.models.score import Score
from pieces_os_client.models.search_engine import SearchEngine
from pieces_os_client.models.search_engines import SearchEngines
from pieces_os_client.models.search_input import SearchInput
from pieces_os_client.models.searched_anchor import SearchedAnchor
from pieces_os_client.models.searched_anchor_point import SearchedAnchorPoint
from pieces_os_client.models.searched_anchor_points import SearchedAnchorPoints
from pieces_os_client.models.searched_anchors import SearchedAnchors
from pieces_os_client.models.searched_annotation import SearchedAnnotation
from pieces_os_client.models.searched_annotations import SearchedAnnotations
from pieces_os_client.models.searched_asset import SearchedAsset
from pieces_os_client.models.searched_assets import SearchedAssets
from pieces_os_client.models.searched_conversation import SearchedConversation
from pieces_os_client.models.searched_conversation_message import SearchedConversationMessage
from pieces_os_client.models.searched_conversation_messages import SearchedConversationMessages
from pieces_os_client.models.searched_conversations import SearchedConversations
from pieces_os_client.models.searched_hint import SearchedHint
from pieces_os_client.models.searched_hints import SearchedHints
from pieces_os_client.models.searched_match_enum import SearchedMatchEnum
from pieces_os_client.models.searched_person import SearchedPerson
from pieces_os_client.models.searched_persons import SearchedPersons
from pieces_os_client.models.searched_sensitive import SearchedSensitive
from pieces_os_client.models.searched_sensitives import SearchedSensitives
from pieces_os_client.models.searched_tag import SearchedTag
from pieces_os_client.models.searched_tags import SearchedTags
from pieces_os_client.models.searched_website import SearchedWebsite
from pieces_os_client.models.searched_websites import SearchedWebsites
from pieces_os_client.models.searched_workstream_pattern_engine_vision_event import SearchedWorkstreamPatternEngineVisionEvent
from pieces_os_client.models.searched_workstream_pattern_engine_vision_events import SearchedWorkstreamPatternEngineVisionEvents
from pieces_os_client.models.searched_workstream_summaries import SearchedWorkstreamSummaries
from pieces_os_client.models.searched_workstream_summary import SearchedWorkstreamSummary
from pieces_os_client.models.seed import Seed
from pieces_os_client.models.seeded_accessor import SeededAccessor
from pieces_os_client.models.seeded_activity import SeededActivity
from pieces_os_client.models.seeded_anchor import SeededAnchor
from pieces_os_client.models.seeded_anchor_point import SeededAnchorPoint
from pieces_os_client.models.seeded_annotation import SeededAnnotation
from pieces_os_client.models.seeded_asset import SeededAsset
from pieces_os_client.models.seeded_asset_enrichment import SeededAssetEnrichment
from pieces_os_client.models.seeded_asset_metadata import SeededAssetMetadata
from pieces_os_client.models.seeded_asset_sensitive import SeededAssetSensitive
from pieces_os_client.models.seeded_asset_tag import SeededAssetTag
from pieces_os_client.models.seeded_asset_tags import SeededAssetTags
from pieces_os_client.models.seeded_asset_website import SeededAssetWebsite
from pieces_os_client.models.seeded_assets_recommendation import SeededAssetsRecommendation
from pieces_os_client.models.seeded_backup import SeededBackup
from pieces_os_client.models.seeded_classification import SeededClassification
from pieces_os_client.models.seeded_connector_asset import SeededConnectorAsset
from pieces_os_client.models.seeded_connector_connection import SeededConnectorConnection
from pieces_os_client.models.seeded_connector_creation import SeededConnectorCreation
from pieces_os_client.models.seeded_connector_tracking import SeededConnectorTracking
from pieces_os_client.models.seeded_conversation import SeededConversation
from pieces_os_client.models.seeded_conversation_message import SeededConversationMessage
from pieces_os_client.models.seeded_discoverable_asset import SeededDiscoverableAsset
from pieces_os_client.models.seeded_discoverable_assets import SeededDiscoverableAssets
from pieces_os_client.models.seeded_discoverable_html_webpage import SeededDiscoverableHtmlWebpage
from pieces_os_client.models.seeded_discoverable_html_webpages import SeededDiscoverableHtmlWebpages
from pieces_os_client.models.seeded_discoverable_related_tag import SeededDiscoverableRelatedTag
from pieces_os_client.models.seeded_discoverable_related_tags import SeededDiscoverableRelatedTags
from pieces_os_client.models.seeded_discoverable_sensitive import SeededDiscoverableSensitive
from pieces_os_client.models.seeded_discoverable_sensitives import SeededDiscoverableSensitives
from pieces_os_client.models.seeded_distribution import SeededDistribution
from pieces_os_client.models.seeded_distributions import SeededDistributions
from pieces_os_client.models.seeded_external_provider import SeededExternalProvider
from pieces_os_client.models.seeded_file import SeededFile
from pieces_os_client.models.seeded_format import SeededFormat
from pieces_os_client.models.seeded_fragment import SeededFragment
from pieces_os_client.models.seeded_git_hub_distribution import SeededGitHubDistribution
from pieces_os_client.models.seeded_git_hub_gist_distribution import SeededGitHubGistDistribution
from pieces_os_client.models.seeded_github_gists_import import SeededGithubGistsImport
from pieces_os_client.models.seeded_hint import SeededHint
from pieces_os_client.models.seeded_mac_os_asset import SeededMacOSAsset
from pieces_os_client.models.seeded_model import SeededModel
from pieces_os_client.models.seeded_models import SeededModels
from pieces_os_client.models.seeded_pkce import SeededPKCE
from pieces_os_client.models.seeded_pkceadditionalparameters import SeededPKCEADDITIONALPARAMETERS
from pieces_os_client.models.seeded_person import SeededPerson
from pieces_os_client.models.seeded_range import SeededRange
from pieces_os_client.models.seeded_range_conversation_association import SeededRangeConversationAssociation
from pieces_os_client.models.seeded_range_conversation_grounding_association import SeededRangeConversationGroundingAssociation
from pieces_os_client.models.seeded_range_conversation_grounding_temporal_association import SeededRangeConversationGroundingTemporalAssociation
from pieces_os_client.models.seeded_score import SeededScore
from pieces_os_client.models.seeded_score_increment import SeededScoreIncrement
from pieces_os_client.models.seeded_sensitive import SeededSensitive
from pieces_os_client.models.seeded_share import SeededShare
from pieces_os_client.models.seeded_tag import SeededTag
from pieces_os_client.models.seeded_tracked_adoption_event import SeededTrackedAdoptionEvent
from pieces_os_client.models.seeded_tracked_application import SeededTrackedApplication
from pieces_os_client.models.seeded_tracked_asset_event import SeededTrackedAssetEvent
from pieces_os_client.models.seeded_tracked_assets_event import SeededTrackedAssetsEvent
from pieces_os_client.models.seeded_tracked_assets_event_metadata import SeededTrackedAssetsEventMetadata
from pieces_os_client.models.seeded_tracked_conversation_event import SeededTrackedConversationEvent
from pieces_os_client.models.seeded_tracked_format_event import SeededTrackedFormatEvent
from pieces_os_client.models.seeded_tracked_interaction_event import SeededTrackedInteractionEvent
from pieces_os_client.models.seeded_tracked_interaction_event_identifier_description_pairs import SeededTrackedInteractionEventIdentifierDescriptionPairs
from pieces_os_client.models.seeded_tracked_keyboard_event import SeededTrackedKeyboardEvent
from pieces_os_client.models.seeded_tracked_keyboard_event_identifier_description_pairs import SeededTrackedKeyboardEventIdentifierDescriptionPairs
from pieces_os_client.models.seeded_tracked_machine_learning_event import SeededTrackedMachineLearningEvent
from pieces_os_client.models.seeded_tracked_session_event import SeededTrackedSessionEvent
from pieces_os_client.models.seeded_ultra_suite_asset import SeededUltraSuiteAsset
from pieces_os_client.models.seeded_user import SeededUser
from pieces_os_client.models.seeded_website import SeededWebsite
from pieces_os_client.models.seeded_workstream_event import SeededWorkstreamEvent
from pieces_os_client.models.seeded_workstream_ingestion import SeededWorkstreamIngestion
from pieces_os_client.models.seeded_workstream_suggestions_refresh import SeededWorkstreamSuggestionsRefresh
from pieces_os_client.models.seeded_workstream_summary import SeededWorkstreamSummary
from pieces_os_client.models.seeds import Seeds
from pieces_os_client.models.segmented_technical_language import SegmentedTechnicalLanguage
from pieces_os_client.models.segmented_technical_language_fragment import SegmentedTechnicalLanguageFragment
from pieces_os_client.models.sensitive import Sensitive
from pieces_os_client.models.sensitive_category_enum import SensitiveCategoryEnum
from pieces_os_client.models.sensitive_metadata import SensitiveMetadata
from pieces_os_client.models.sensitive_severity_enum import SensitiveSeverityEnum
from pieces_os_client.models.sensitives import Sensitives
from pieces_os_client.models.session import Session
from pieces_os_client.models.share import Share
from pieces_os_client.models.shares import Shares
from pieces_os_client.models.space import Space
from pieces_os_client.models.streamed_identifier import StreamedIdentifier
from pieces_os_client.models.streamed_identifiers import StreamedIdentifiers
from pieces_os_client.models.suggestion import Suggestion
from pieces_os_client.models.suggestion_target import SuggestionTarget
from pieces_os_client.models.system_execution_cpu_information import SystemExecutionCpuInformation
from pieces_os_client.models.system_execution_information import SystemExecutionInformation
from pieces_os_client.models.tlp_code_directory_analytics import TLPCodeDirectoryAnalytics
from pieces_os_client.models.tlp_code_file_analytics import TLPCodeFileAnalytics
from pieces_os_client.models.tlp_code_fragment_classification import TLPCodeFragmentClassification
from pieces_os_client.models.tlp_code_fragment_classification_metadata import TLPCodeFragmentClassificationMetadata
from pieces_os_client.models.tlp_code_fragment_description import TLPCodeFragmentDescription
from pieces_os_client.models.tlp_code_fragment_descriptive_statistics import TLPCodeFragmentDescriptiveStatistics
from pieces_os_client.models.tlp_code_fragment_reclassification import TLPCodeFragmentReclassification
from pieces_os_client.models.tlp_code_fragment_reclassification_updates import TLPCodeFragmentReclassificationUpdates
from pieces_os_client.models.tlp_code_fragment_statistics import TLPCodeFragmentStatistics
from pieces_os_client.models.tlp_code_fragment_suggested_reuse import TLPCodeFragmentSuggestedReuse
from pieces_os_client.models.tlp_code_fragment_suggested_save import TLPCodeFragmentSuggestedSave
from pieces_os_client.models.tlp_code_fragment_tagify import TLPCodeFragmentTagify
from pieces_os_client.models.tlp_code_processing import TLPCodeProcessing
from pieces_os_client.models.tlp_code_repository_analytics import TLPCodeRepositoryAnalytics
from pieces_os_client.models.tlp_code_snippet_analytics import TLPCodeSnippetAnalytics
from pieces_os_client.models.tlp_code_snippet_suggested_interactions import TLPCodeSnippetSuggestedInteractions
from pieces_os_client.models.tlp_code_snippet_tagify_code import TLPCodeSnippetTagifyCode
from pieces_os_client.models.tlp_directed_discovery_filter import TLPDirectedDiscoveryFilter
from pieces_os_client.models.tlp_directed_discovery_filter_enum import TLPDirectedDiscoveryFilterEnum
from pieces_os_client.models.tlp_directed_discovery_filters import TLPDirectedDiscoveryFilters
from pieces_os_client.models.tlp_machine_learning_processing_event import TLPMachineLearningProcessingEvent
from pieces_os_client.models.tag import Tag
from pieces_os_client.models.tag_category_enum import TagCategoryEnum
from pieces_os_client.models.tags import Tags
from pieces_os_client.models.temporal_range_grounding import TemporalRangeGrounding
from pieces_os_client.models.temporal_search_options import TemporalSearchOptions
from pieces_os_client.models.terminating_os_server_applet import TerminatingOSServerApplet
from pieces_os_client.models.text_location import TextLocation
from pieces_os_client.models.text_match import TextMatch
from pieces_os_client.models.textually_extracted_material import TextuallyExtractedMaterial
from pieces_os_client.models.textually_extracted_materials import TextuallyExtractedMaterials
from pieces_os_client.models.theme import Theme
from pieces_os_client.models.tokenized_pkce import TokenizedPKCE
from pieces_os_client.models.tracked_asset_event_creation_metadata import TrackedAssetEventCreationMetadata
from pieces_os_client.models.tracked_asset_event_creation_metadata_clipboard import TrackedAssetEventCreationMetadataClipboard
from pieces_os_client.models.tracked_asset_event_creation_metadata_file import TrackedAssetEventCreationMetadataFile
from pieces_os_client.models.tracked_asset_event_format_reclassification_metadata import TrackedAssetEventFormatReclassificationMetadata
from pieces_os_client.models.tracked_asset_event_identifier_description_pairs import TrackedAssetEventIdentifierDescriptionPairs
from pieces_os_client.models.tracked_asset_event_metadata import TrackedAssetEventMetadata
from pieces_os_client.models.tracked_asset_event_rename_metadata import TrackedAssetEventRenameMetadata
from pieces_os_client.models.tracked_assets_event_identifier_description_pairs import TrackedAssetsEventIdentifierDescriptionPairs
from pieces_os_client.models.tracked_assets_event_search_metadata import TrackedAssetsEventSearchMetadata
from pieces_os_client.models.tracked_assets_event_search_metadata_results import TrackedAssetsEventSearchMetadataResults
from pieces_os_client.models.tracked_conversation_event_identifier_description_pairs import TrackedConversationEventIdentifierDescriptionPairs
from pieces_os_client.models.tracked_conversation_event_metadata import TrackedConversationEventMetadata
from pieces_os_client.models.tracked_conversation_event_rename_metadata import TrackedConversationEventRenameMetadata
from pieces_os_client.models.tracked_format_event_identifier_description_pairs import TrackedFormatEventIdentifierDescriptionPairs
from pieces_os_client.models.tracked_format_event_metadata import TrackedFormatEventMetadata
from pieces_os_client.models.tracked_session_event_identifier_description_pairs import TrackedSessionEventIdentifierDescriptionPairs
from pieces_os_client.models.tracked_summary_totals import TrackedSummaryTotals
from pieces_os_client.models.transferable_bytes import TransferableBytes
from pieces_os_client.models.transferable_string import TransferableString
from pieces_os_client.models.unchecked_os_server_update import UncheckedOSServerUpdate
from pieces_os_client.models.unsegmented_technical_language import UnsegmentedTechnicalLanguage
from pieces_os_client.models.updated_external_provider_api_key import UpdatedExternalProviderApiKey
from pieces_os_client.models.updating_status_enum import UpdatingStatusEnum
from pieces_os_client.models.user_beta_status import UserBetaStatus
from pieces_os_client.models.user_profile import UserProfile
from pieces_os_client.models.users import Users
from pieces_os_client.models.verified_os_filesystem_path import VerifiedOSFilesystemPath
from pieces_os_client.models.website import Website
from pieces_os_client.models.websites import Websites
from pieces_os_client.models.window_dimensions import WindowDimensions
from pieces_os_client.models.workstream_event import WorkstreamEvent
from pieces_os_client.models.workstream_event_context import WorkstreamEventContext
from pieces_os_client.models.workstream_event_trigger import WorkstreamEventTrigger
from pieces_os_client.models.workstream_event_trigger_context_browser import WorkstreamEventTriggerContextBrowser
from pieces_os_client.models.workstream_event_trigger_context_ide import WorkstreamEventTriggerContextIDE
from pieces_os_client.models.workstream_events import WorkstreamEvents
from pieces_os_client.models.workstream_ingestion import WorkstreamIngestion
from pieces_os_client.models.workstream_pattern_engine_data_cleanup_request import WorkstreamPatternEngineDataCleanupRequest
from pieces_os_client.models.workstream_pattern_engine_source import WorkstreamPatternEngineSource
from pieces_os_client.models.workstream_pattern_engine_sources import WorkstreamPatternEngineSources
from pieces_os_client.models.workstream_pattern_engine_status import WorkstreamPatternEngineStatus
from pieces_os_client.models.workstream_pattern_engine_vision_calibration import WorkstreamPatternEngineVisionCalibration
from pieces_os_client.models.workstream_pattern_engine_vision_calibrations import WorkstreamPatternEngineVisionCalibrations
from pieces_os_client.models.workstream_pattern_engine_vision_event import WorkstreamPatternEngineVisionEvent
from pieces_os_client.models.workstream_pattern_engine_vision_event_deletions import WorkstreamPatternEngineVisionEventDeletions
from pieces_os_client.models.workstream_pattern_engine_vision_event_textual_value import WorkstreamPatternEngineVisionEventTextualValue
from pieces_os_client.models.workstream_pattern_engine_vision_events import WorkstreamPatternEngineVisionEvents
from pieces_os_client.models.workstream_pattern_engine_vision_events_metadata import WorkstreamPatternEngineVisionEventsMetadata
from pieces_os_client.models.workstream_pattern_engine_vision_metadata import WorkstreamPatternEngineVisionMetadata
from pieces_os_client.models.workstream_pattern_engine_vision_status import WorkstreamPatternEngineVisionStatus
from pieces_os_client.models.workstream_search_options import WorkstreamSearchOptions
from pieces_os_client.models.workstream_suggestion import WorkstreamSuggestion
from pieces_os_client.models.workstream_suggestion_type import WorkstreamSuggestionType
from pieces_os_client.models.workstream_suggestions import WorkstreamSuggestions
from pieces_os_client.models.workstream_suggestions_refresh import WorkstreamSuggestionsRefresh
from pieces_os_client.models.workstream_summaries import WorkstreamSummaries
from pieces_os_client.models.workstream_summary import WorkstreamSummary



import threading
from pieces_os_client import UserProfile,AllocationStatusEnum
from typing import Optional

## TODO: Modify the Basic class to be able to fit in the BasicUser
class BasicUser:
	"""
	A class to represent a basic user and manage their connection to the cloud.

	Attributes:
		user_profile: The profile of the user.
		pieces_client: The client used to interact with the pieces OS API.
	"""

	user_profile: Optional[UserProfile] = None

	def __init__(self, pieces_client) -> None:
		"""
		Initializes the BasicUser with a pieces client.

		Args:
			pieces_client: The client used to interact with the pieces OS API.
		"""
		self.pieces_client = pieces_client

	def on_user_callback(self, user: Optional[UserProfile], connecting=False):
		"""
		Callback function to set the user profile.

		Args:
			user: The profile of the user.
			connecting: A flag indicating if the user is connecting to the cloud (default is False).
		"""
		self.user_profile = user

	def _on_login_connect(self, thread, timeout):
		"""
		Waits for the user to login and then connects to the cloud.

		Args:
			thread: The thread handling the login process.
			timeout: The maximum time to wait for the login process.
		"""
		thread.get(timeout)  # Wait for the user to login
		self.connect()

	def login(self, connect_after_login=True, timeout=120):
		"""
		Logs the user into the OS and optionally connects to the cloud.

		Args:
			connect_after_login: A flag indicating if the user should connect to the cloud after login (default is True).
			timeout: The maximum time to wait for the login process (default is 120 seconds).
		"""
		thread = self.pieces_client.os_api.sign_into_os(async_req=True)
		if connect_after_login:
			threading.Thread(target=lambda: self._on_login_connect(thread, timeout))

	def logout(self):
		"""
		Logs the user out of the OS.
		"""
		self.pieces_client.api_client.os_api.sign_out_of_os()

	def connect(self):
		"""
		Connects the user to the cloud.

		Raises:
			PermissionError: If the user is not logged in.
		"""
		if not self.user_profile:
			raise PermissionError("You must be logged in to use this feature")
		self.on_user_callback(self.user_profile, True)  # Set the connecting to cloud bool to true
		self.pieces_client.allocations_api.allocations_connect_new_cloud(self.user_profile)

	def disconnect(self):
		"""
		Disconnects the user from the cloud.

		Raises:
			PermissionError: If the user is not logged in.
		"""
		if not self.user_profile:
			raise PermissionError("You must be logged in to use this feature")
		if self.user_profile.allocation:  # Check if there is an allocation iterable
			self.pieces_client.api_client.allocations_api.allocations_disconnect_cloud(self.user_profile.allocation)

	@property
	def picture(self) -> Optional[str]:
		"""
		Returns the user's profile picture URL.

		Returns:
			The URL of the user's profile picture, or None if not available.
		"""
		if self.user_profile:
			return self.user_profile.picture


	@property
	def name(self) -> Optional[str]:
		"""
		Returns the name of the user.

		Returns:
			Optional[str]: The name of the user if the user logged in, otherwise None.
		"""
		if self.user_profile:
			return self.user_profile.name

	@property
	def email(self) -> Optional[str]:
		"""
		Returns the email of the user.

		Returns:
			Optional[str]: The email of the user if the user logged in, otherwise None.
		"""
		if self.user_profile:
			return self.user_profile.email

	@property
	def vanity_name(self) -> Optional[str]: # TODO: implements the setter object
		"""
		Returns the vanity name of the user which is the base name of the cloud url.
			For example, if the cloud URL is 'bishoyatpieces.pieces.cloud', this method returns 'bishoyatpieces'.

		Returns:
			Optional[str]: The vanity name of the user if the user user logged in and connected to the cloud, otherwise None.
		"""
		if self.user_profile:
			return self.user_profile.vanityname

	@property
	def cloud_status(self) -> Optional[AllocationStatusEnum]:
		"""
		Returns the cloud status of the user's cloud.

		Returns:
			Optional[AllocationStatusEnum]: The cloud status of the user's cloud.
		"""
		if self.user_profile and self.user_profile.allocation:
			return self.user_profile.allocation.status.cloud



# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from pieces_os_client.models.classification import Classification
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.grouped_timestamp import GroupedTimestamp
from pieces_os_client.models.person_basic_type import PersonBasicType
from pieces_os_client.models.transferable_string import TransferableString

class DocumentContributor(BaseModel):
    """
    A DocumentContributor is a preseeded representation of a Person  This can be used in the case of the browser, or in the IDE  If they are apart of an IDE, we can in the future provide git information (IE add a Git object for their commits)  person: this is most important part which is the email/name xyz  # noqa: E501
    """
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    classification: Optional[Classification] = None
    value: Optional[TransferableString] = None
    person: PersonBasicType = Field(...)
    updated: Optional[GroupedTimestamp] = None
    __properties = ["schema", "classification", "value", "person", "updated"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> DocumentContributor:
        """Create an instance of DocumentContributor from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of classification
        if self.classification:
            _dict['classification'] = self.classification.to_dict()
        # override the default output from pydantic by calling `to_dict()` of value
        if self.value:
            _dict['value'] = self.value.to_dict()
        # override the default output from pydantic by calling `to_dict()` of person
        if self.person:
            _dict['person'] = self.person.to_dict()
        # override the default output from pydantic by calling `to_dict()` of updated
        if self.updated:
            _dict['updated'] = self.updated.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> DocumentContributor:
        """Create an instance of DocumentContributor from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return DocumentContributor.parse_obj(obj)

        _obj = DocumentContributor.parse_obj({
            "var_schema": EmbeddedModelSchema.from_dict(obj.get("schema")) if obj.get("schema") is not None else None,
            "classification": Classification.from_dict(obj.get("classification")) if obj.get("classification") is not None else None,
            "value": TransferableString.from_dict(obj.get("value")) if obj.get("value") is not None else None,
            "person": PersonBasicType.from_dict(obj.get("person")) if obj.get("person") is not None else None,
            "updated": GroupedTimestamp.from_dict(obj.get("updated")) if obj.get("updated") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from pieces_os_client.models.embedded_model_schema import EmbeddedModelSchema
from pieces_os_client.models.model_download_progress_status_enum import ModelDownloadProgressStatusEnum
from pieces_os_client.models.transferable_bytes import TransferableBytes
from typing import Optional, Set
from typing_extensions import Self

class OSFileReadStreamedProgress(BaseModel):
    """
    This is a model to return stream progress for a file read.
    """ # noqa: E501
    var_schema: Optional[EmbeddedModelSchema] = Field(default=None, alias="schema")
    status: ModelDownloadProgressStatusEnum
    percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Optionally if the download is in progress you will receive a download percent(from 0-100).")
    path: StrictStr
    id: StrictStr = Field(description="This is a generated UUID that represents this current stream in progress(can be used to cancel this in the future)")
    bytes: Optional[TransferableBytes] = None
    __properties: ClassVar[List[str]] = ["schema", "status", "percentage", "path", "id", "bytes"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OSFileReadStreamedProgress from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of var_schema
        if self.var_schema:
            _dict['schema'] = self.var_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bytes
        if self.bytes:
            _dict['bytes'] = self.bytes.to_dict()
        # set to None if percentage (nullable) is None
        # and model_fields_set contains the field
        if self.percentage is None and "percentage" in self.model_fields_set:
            _dict['percentage'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OSFileReadStreamedProgress from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "schema": EmbeddedModelSchema.from_dict(obj["schema"]) if obj.get("schema") is not None else None,
            "status": obj.get("status"),
            "percentage": obj.get("percentage"),
            "path": obj.get("path"),
            "id": obj.get("id"),
            "bytes": TransferableBytes.from_dict(obj["bytes"]) if obj.get("bytes") is not None else None
        })
        return _obj





from typing import Callable, Optional,TYPE_CHECKING
from typing_extensions import Self
import websocket
import threading
from abc import ABC, abstractmethod

if TYPE_CHECKING:
	from ..client import PiecesClient

class BaseWebsocket(ABC):
	instances = []
	_initialized_events:list[threading.Event] = []

	def __new__(cls, *args, **kwargs):
		"""
		Ensure that only one instance of the url class is created (Singleton pattern).
		"""
		if not hasattr(cls, 'instance'):
			cls.instance = super(BaseWebsocket, cls).__new__(cls)
		return cls.instance

	def __init__(self,
				 pieces_client: "PiecesClient",
				 on_message_callback: Callable[[str], None],
				 on_open_callback: Optional[Callable[[websocket.WebSocketApp], None]] = None,
				 on_error: Optional[Callable[[websocket.WebSocketApp, Exception], None]] = None,
				 on_close: Optional[Callable[[websocket.WebSocketApp], None]] = None):
		"""
		Initialize the BaseWebsocket instance.

		:param pieces_client: An instance of PiecesClient.
		:param on_message_callback: Callback function to handle incoming messages.
		:param on_open_callback: Optional callback function to handle the websocket opening.
		:param on_error: Optional callback function to handle errors.
		:param on_close: Optional callback function to handle the websocket closing.
		"""
		self.ws = None
		self.thread = None
		self.running = False
		self.on_message_callback = on_message_callback
		self.on_open_callback = on_open_callback if on_open_callback else lambda x: None
		self.on_error = on_error if on_error else lambda ws, error: print(error)
		self.on_close = on_close if on_close else lambda ws, close_status_code, close_msg: None
		self.pieces_client = pieces_client
		self._initialized = threading.Event()
		self._initialized.set() # Set it as true at the beginning


		if self not in BaseWebsocket.instances:
			BaseWebsocket.instances.append(self)
			BaseWebsocket._initialized_events.append(self._initialized)
	
	@property
	def _is_initialized_on_open(self):
		return True

	@abstractmethod
	def url(self) -> str:
		"""
		The URL to connect to. Should be overridden by subclasses.
		"""
		pass

	@abstractmethod
	def on_message(self, ws, message):
		"""
		Handle incoming messages. Should be overridden by subclasses.
		"""
		pass

	def on_open(self, ws):
		"""
		Handle the websocket opening event.
		"""
		self.running = True
		self.on_open_callback(ws)
		if self._is_initialized_on_open:
			self._initialized.set()

	def run(self):
		"""
		Run the websocket connection.
		"""
		self.ws = websocket.WebSocketApp(
			self.url,
			on_message=self.on_message,
			on_error=self.on_error,
			on_close=self.on_close,
			on_open=self.on_open
		)
		self.ws.run_forever()

	def start(self):
		"""
		Start the websocket connection in a new thread.
		"""
		if not self.running:
			self._initialized.clear()
			self.thread = threading.Thread(target=self.run)
			self.thread.start()

	def close(self):
		"""
		Close the websocket connection and stop the thread.
		"""
		if self.running:
			self.ws.close()
			self.thread.join()
			self.running = False

	@classmethod
	def close_all(cls):
		"""
		Close all websocket instances.
		"""
		for instance in cls.instances:
			instance.close()

	@classmethod
	def reconnect_all(cls):
		"""
		Reconnect all websocket instances.
		"""
		for instance in cls.instances:
			instance.reconnect()

	def reconnect(self):
		"""
		Reconnect the websocket connection.
		"""
		self.close()
		self.start()

	def __str__(self):
		"""
		Return a string representation of the instance.
		"""
		return getattr(self, "url", "BaseWebsocket(ABC)")

	@classmethod
	def is_running(cls) -> bool:
		"""
		Check if the websocket instance is running.

		:return: True if running, False otherwise.
		"""
		instance = cls.get_instance()
		if instance:
			return cls.instance.running
		return False

	@classmethod
	def get_instance(cls) -> Optional[type]:
		"""
		Get the singleton instance of the class.

		:return: The singleton instance or None if not created.
		"""
		return getattr(cls, 'instance', None)

	@classmethod
	def start_all(cls):
		"""
		Start all the websockets that are already initialized.
		"""
		for ws in cls.instances:
			ws.start()

	@classmethod
	def wait_all(cls):
		"""
			Wait for all websockets to set the internal flag on
		"""
		for event in cls._initialized_events:
			event.wait()




# coding: utf-8

"""
    Pieces Isomorphic OpenAPI

    Endpoints for Assets, Formats, Users, Asset, Format, User.

    The version of the OpenAPI document: 1.0
    Contact: tsavo@pieces.app
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ExternalProviderTypeEnum(str, Enum):
    """
    This is an enum to describe the provider used to authenticate a user.  IMPORTANT NOT to modify tha values here unless they are explicitly mapped with auth0's provider values(example within cloud server)  'waad': (windows azure active directory)
    """

    """
    allowed enum values
    """
    GITHUB = 'github'
    AUTH0 = 'auth0'
    GOOGLE_MINUS_OAUTH2 = 'google-oauth2'
    BITBUCKET = 'bitbucket'
    LINKEDIN = 'linkedin'
    TWITTER = 'twitter'
    FACEBOOK = 'facebook'
    WAAD = 'waad'

    @classmethod
    def from_json(cls, json_str: str) -> ExternalProviderTypeEnum:
        """Create an instance of ExternalProviderTypeEnum from a JSON string"""
        return ExternalProviderTypeEnum(json.loads(json_str))





from enum import Enum
import re
from typing import Optional

class VersionChecker:
    def __init__(self, min_version: str, max_version: str, pieces_os_version:str):
        self.min_version = min_version
        self.max_version = max_version
        self.pieces_os_version = pieces_os_version

    @staticmethod
    def _parse_version(version_str):
        """Parse a version string into a tuple of integers and pre-release labels."""
        match = re.match(r'^(\d+)\.(\d+)\.(\d+)(?:[-.](\S+))?$', version_str)
        if not match:
            raise ValueError(f"Invalid version format: {version_str}")

        major, minor, patch, pre_release = match.groups()
        version_tuple = (int(major), int(minor), int(patch))
        pre_release_tuple = tuple(pre_release.split('.')) if pre_release else ()
        return version_tuple, pre_release_tuple

    def version_check(self):
        """Check if the Pieces OS version is within the supported range."""
        # Parse version numbers
        os_version_parsed, os_pre_release = self._parse_version(self.pieces_os_version)
        min_version_parsed, min_pre_release = self._parse_version(self.min_version)
        max_version_parsed, max_pre_release = self._parse_version(self.max_version)

        # Determine compatibility
        if (os_version_parsed < min_version_parsed or
            (os_version_parsed == min_version_parsed and os_pre_release < min_pre_release)):
            return VersionCheckResult(False, UpdateEnum.PiecesOS)
        elif (os_version_parsed > max_version_parsed or
              (os_version_parsed == max_version_parsed and os_pre_release >= max_pre_release)):
            return VersionCheckResult(False, UpdateEnum.Plugin)

        return VersionCheckResult(True)


class UpdateEnum(Enum):
    PiecesOS = 1
    Plugin = 2

class VersionCheckResult:
    """Result of the version check."""
    def __init__(self, compatible, update:Optional[UpdateEnum]=None):
        self.compatible = compatible
        self.update = update
